=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-24 04:40:06 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-08-07 10:59:30 +0000
@@ -278,8 +278,10 @@
     def test_set_last_revision(self):
         wt = self.make_branch_and_tree('source')
         self.assertEqual(None, wt.last_revision())
-        # cannot set the last revision to one not in the branch history.
-        self.assertRaises(errors.NoSuchRevision, wt.set_last_revision, 'A')
+        # set last-revision to one not in the history
+        wt.set_last_revision('A')
+        # set it back to None for an empty tree.
+        wt.set_last_revision(None)
         wt.commit('A', allow_pointless=True, rev_id='A')
         self.assertEqual('A', wt.last_revision())
         # None is aways in the branch

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-07-29 14:37:24 +0000
+++ bzrlib/workingtree.py	2006-08-07 10:59:30 +0000
@@ -1224,13 +1224,11 @@
         if new_revision is None:
             self.branch.set_revision_history([])
             return False
-        # current format is locked in with the branch
-        revision_history = self.branch.revision_history()
         try:
-            position = revision_history.index(new_revision)
-        except ValueError:
-            raise errors.NoSuchRevision(self.branch, new_revision)
-        self.branch.set_revision_history(revision_history[:position + 1])
+            self.branch.generate_revision_history(new_revision)
+        except errors.NoSuchRevision:
+            # not present in the repo - dont try to set it deeper than the tip
+            self.branch.set_revision_history([new_revision])
         return True
 
     def _cache_basis_inventory(self, new_revision):
@@ -1259,7 +1257,7 @@
             path = self._basis_inventory_name()
             sio = StringIO(xml)
             self._control_files.put(path, sio)
-        except WeaveRevisionNotPresent:
+        except (errors.NoSuchRevision, errors.RevisionNotPresent):
             pass
 
     def read_basis_inventory(self):
@@ -1527,10 +1525,6 @@
                 pass
             return False
         else:
-            try:
-                self.branch.revision_history().index(revision_id)
-            except ValueError:
-                raise errors.NoSuchRevision(self.branch, revision_id)
             self._control_files.put_utf8('last-revision', revision_id)
             return True
 

